## analysis of children's production (the "says" analysis, 15 month olds)

require(lme4)

data.say <- read.table("data_public/dat_crop_says.txt", header=T, sep="\t")
data.say$says.15  <- as.factor(data.say$says.15)
data.say$word.category <- relevel(data.say$word.category, ref="noun")

######## top analysis ######

says.final.glm <- glmer(data= data.say, 
                        says.15 ~ total.freq.log.c + isol.freq.log.c + 
                          mlu.mean.imputed + duration.ratio.c + word.category +
                          (1+word.category|mom) + (1|word), 
                        family="binomial",
                        control=glmerControl(optimizer="bobyqa", optCtrl=list(maxfun=100000))
)

##------------------------------------##

##----------------##

## imputation

## use functions from here:
source('recreate_public/functions_say.R')

## use mean,sd values for imputations from here:
source('recreate_public/meansdlist_say.R')
## you'll also need these for the permutation analysis to follow 


say_formula <- as.formula('says.15 ~ total.freq.log.c + isol.freq.log.c + 
                          mlu.imputed.c + duration.ratio.c + word.category +
                          (1+word.category|mom) + (1|word)')

reglist <- list()
M = 100

reglist <- imputeMLU(
    dat = data.say,
    model.formula = say_formula,
    mluname = 'mlu.before.impute',
    M.imputes = M,
    meansdlist = say_meansdlist,
    print_each_loop = TRUE)

## now combine

tempcoef = lapply(reglist, function(x){summary(x)$coefficients})
templik = lapply(reglist, logLik)

## remove failed examples, hopefully none
ind = unlist(lapply(tempcoef, function(x){sum(is.na(x))})) == 0
## gather coefs

coefs.gather = do.call(cbind, lapply(tempcoef[ind], function(x){x[,1]}))
sds.gather = do.call(cbind, lapply(tempcoef[ind], function(x){x[,2]}))

coefs.mean = rowMeans(coefs.gather)

temp.b = coefs.gather - coefs.mean

M = sum(ind)

coefs.sds = sqrt(rowSums(sds.gather^2) / M + (1 + 1/M) * (1 / (M - 1)) * rowSums(temp.b^2))

df.say = sum(!is.na(data.say$duration.ratio.c) & !is.na(data.say$mlu.before.impute)) ## approximation, but v close
coefs.impute <- cbind(coefs.mean,
                      coefs.sds,
                      coefs.mean / coefs.sds,
                      2 * pt(-abs(coefs.mean / coefs.sds), df = df.say))


colnames(coefs.impute) <- colnames(summary(says.final.glm)$coefficients)

##------------------------------------##

## permutation analysis

## this code will take a while time to run,
## [at least in 2016...]
## although compared to the ordinal analysis, it's really not so bad...
## we ran it on a large distributed cluster, not a single machine
## you can test run a small number of permutations as long as you also
## use a small number of imputations

library(gtools)

## for us, we have seven mothers to permute
## perm matrix
perm7 <- permutations(n = 7, r = 7)
                                        # paste perms together for naming them 
temp <- lapply(1:7, function(x){perm7[,x]})
perms <- do.call(paste0, temp)

                                        # change matrix to list with names,
                                        # to make it easier to call perms by name
permlist <- lapply(1:factorial(7), function(x){perm7[x,]})
names(permlist) <- perms

##----------------##

## which permutations do you want to use?

## all
permloop = perms

## if you randomly selected your perms, you might want to set the first as the
## truth for easier analysis
## permloop[1] = perms[1]

perm.var = 'says.15' ## our Y
M = 20 ## number of multiple imputations for each loop
## remember: however long the first regression took (clmm.final),
## this loop will take {length(permloop) * M} times more time

## plan is that for each permutation, we do a multiple imputation,
## generate the relevant coefficient matrix, and store that in this list:
coefs.list <- list()

## depending on your model, some may fail to converge with some permutations!
for(permname in permloop){

    perm = permlist[[permname]]
    print(perm) ## delete if you don't want to see the print

    ## for us, the data comes in stacks of 7,
    ## so we just permute within each block of 7 rows
    datind <- rep((0:(nrow(data.say)/7 - 1))*7, each = 7)
    datind = datind + rep(perm, times = nrow(data.say)/7)

    ## create a new instance of the data
    perm.dat = data.say
    ## this is the permutation
    perm.dat[,perm.var] <- data.say[datind,perm.var]

    ## perform the regression, with imputation
    reglist <- imputeMLU(
        dat = perm.dat,
        model.formula = say_formula,
        mluname = 'mlu.before.impute',
        M.imputes = M,
        meansdlist = say_meansdlist,
        print_each_loop = TRUE)
    

    ## now combine

    tempcoef = lapply(reglist, function(x){summary(x)$coefficients})
    templik = lapply(reglist, logLik)

    ## remove failed examples, hopefully none
    ind = unlist(lapply(tempcoef, function(x){sum(is.na(x))})) == 0
    ## gather coefs

    coefs.gather = do.call(cbind, lapply(tempcoef[ind], function(x){x[,1]}))
    sds.gather = do.call(cbind, lapply(tempcoef[ind], function(x){x[,2]}))

    coefs.mean = rowMeans(coefs.gather)

    temp.b = coefs.gather - coefs.mean

    M.ind = sum(ind)

    coefs.sds = sqrt(rowSums(sds.gather^2) / M.ind +
                         (1 + 1/M.ind) * (1 / (M.ind - 1)) * rowSums(temp.b^2))

    df_und = sum(!is.na(orig_mlu)) ## an approximation, but v close
    coefs.impute <- cbind(coefs.mean,
                          coefs.sds,
                          coefs.mean / coefs.sds,
                          2 * pt(-abs(coefs.mean / coefs.sds), df = df_und))

    anyreg <- lm(c(1,2,3) ~ c(1,1,2)) ## just for the names!
    colnames(coefs.impute) <- colnames(summary(anyreg)$coefficients)

    coefs.list[[permname]] = coefs.impute
}

##----------------##

## analyse the permutation analysis:
## (at least some stuff you might want to do)

ests.list <- lapply(coefs.list, function(x){x[,1]})
pval.list <- lapply(coefs.list, function(x){x[,4]})

ests.pre <- do.call(rbind, ests.list)
ests <- ests.pre[,2:ncol(ests.pre)] # drop intercept

pval.pre <- do.call(rbind, pval.list)
pval <- pval.pre[,2:ncol(pval.pre)]

## if we set the first loop as the identity, we can just check the first value against the rest
## how many are 'better' that the truth?
## (i.e. how many p-values are smaller than the identity permutation p-values)
pvalquantiles <- apply(pval, 2, function(x){sum(x < x[1])}) / (nrow(pval)-1)
pvalquantiles

